/********************************************************************
 * messages.c
 *
 * This work is licensed under a Creative Commons Attribution-Share Alike 3.0
 * United States License. See http://creativecommons.org/licenses/by-sa/3.0/us/
 * for details.
 *
 * This file is part of thinkfan. See thinkfan.c for further info.
 * ******************************************************************/

#ifndef MESSAGES_C

#define MESSAGES_C

#define MSG_USAGE \
 "\nthinkfan " VERSION ": A minimalist fan control program\n" \
 "\nUsage: thinkfan [-hnqzD [-b BIAS] [-c CONFIG] [-s SECONDS]]" \
 "\n -h  This help message" \
 "\n -s  Maximum seconds between temperature updates (default: 5)" \
 "\n -b  Floating point number (0 <= BIAS <= 20) to control rising edge" \
 "\n     temperature biasing strength (see README). Default 5." \
 "\n -c  Load different configuration file (default: /etc/thinkfan.conf)" \
 "\n -n  Do not become a daemon and log to terminal & syslog" \
 "\n -q  Be quiet (no status info on terminal)" \
 "\n -z  Assume we don't have to worry about resuming when using the sysfs" \
 "\n     interface (see README!)" \
 "\n -D  DANGEROUS mode: Disable all sanity checks. May damage your " \
 "hardware!!\n\n"

#define MSG_DBG_T_STAT "sleeptime=%d, temp=%d, last_temp=%d, biased_temp=%d" \
 " -> level=%d\n", st, temp, last_temp, b_temp, config->items[lvl_idx].level
#define MSG_DBG_CONF_RELOAD "Received SIGHUP: reloading config...\n"

#define MSG_INF_SANITY "Sanity checks are on. Exiting.\n"
#define MSG_INF_INSANITY "Sanity checks are off. Continuing.\n"
#define MSG_INF_CONFIG \
 "Config as read from %s:\nFan level\tLow\tHigh\n", config_file
#define MSG_INF_CONF_ITEM(level, low, high) " %d\t\t%d\t%d\n", level, low, high
#define MSG_INF_TERM \
 "Cleaning up and resetting fan control.\n"
#define MSG_WRN_SLEEPTIME_15 "WARNING: %d seconds of not realizing " \
 "rising temperatures may be dangerous!\n", sleeptime
#define MSG_WRN_SLEEPTIME_1 "A sleeptime of %d seconds doesn't make much " \
 "sense.\n", sleeptime
#define MSG_ERR_T_GET "Error getting temperature.\n"
#define MSG_ERR_MODOPTS \
 "Module thinkpad_acpi doesn't seem to support fan_control\n"
#define MSG_ERR_FAN_WRITE "Error writing to " IBM_FAN ".\n"
#define MSG_ERR_FAN_INIT "Error initializing fan watchdog.\n"
#define MSG_ERR_OPT_S "ERROR: option -s requires an int argument!\n"
#define MSG_ERR_OPT_B "ERROR: option -b requires a float argument!\n"
#define MSG_ERR_ERRCOUNT "Counted %d error(s) during fancontrol. " \
	"Exiting.\n", ret
#define MSG_ERR_CONF_NOFILE "Refusing to run without usable config file." \
 " Please read the documentation.\n"
#define MSG_ERR_CONF_LOST "Lost configuration! This is a bug. Please " \
 "report this to the author.\n"
#define MSG_ERR_CONF_RELOAD "Error reloading config. Keeping old one.\n"
#define MSG_ERR_CONF_PARSE "Empty config or wrong syntax!\n"
#define MSG_ERR_CONF_ORDER "%s:%d: Fan level needs to be higher than the " \
	"previous one!\n", fname, line_count
#define MSG_ERR_CONF_LOWHIGH "%s:%d: LOWER limit must be smaller than HIGHER" \
	"(duh!)\n... Well really, don't mess with this, it could trash your " \
	"hardware.\n", fname, line_count
#define MSG_ERR_RUNNING PID_FILE " already exists. Either thinkfan is " \
	"already running, or it was killed by SIGKILL. If you're sure thinkfan" \
	" is not running, delete " PID_FILE " manually.\n"
#define MSG_WRN_CONF_FAN "WARNING: Found an additional \"fan\" directive " \
	"in config file.\n"
#define MSG_WRN_SYSFS_SAFE "Using safe but wasteful way of setting PWM value.\n"


#endif
