/*************************************************************************
 * thinkfan version 0.5 -- copyleft 06-2009, Victor Mataré
 *
 * This work is licensed under a Creative Commons Attribution-Share Alike 3.0
 * United States License. See http://creativecommons.org/licenses/by-sa/3.0/us/
 * for details.
 *
 * A minimalist, lightweight fan control program for modern Thinkpads.
 *
 * This program is provided to you AS-IS. You can do with it whatever
 * you want. No warranties whatsoever. If this program steals your
 * car, kills your horse, smokes your dope or pees on your carpet...
 * - too bad, you're on your own.
 *
 *   !!!.. WARNING ..!!!.. WARNING ..!!!.. WARNING ..!!!................
 *   !!!
 *   !!!    This program could damage your hardware in any thinkable
 *   !!!    way. Most likely it will OVERHEAT YOUR CPU.
 *   !!!    There is NO SANITY CHECKING on the configuration.
 *   !!!    That means you can configure it to put your CPU on fire.
 *   !!!
 *   !!!    Don't run it if you don't understand what it does.
 *
 *
 *   You have been warned.
 *
 *************************************************************************/

#define _GNU_SOURCE

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include "tf_system.c"


/*********************************************************
 * readconfig(char *fname) reads the config file and
 * returns a struct tf_config.
 * Non-matching lines are skipped.
 *********************************************************/
tf_config *readconfig(char* fname) {
	FILE *cfg_file;
	int i=0, line_count=0, sensor_count=0;
	ssize_t line_len;
	size_t ll;
	tf_config *cfg_local;
	char *line=NULL;
	char buf[1024];

	cfg_local = (tf_config *) malloc(sizeof(tf_config));
	cfg_local->items = (thm_tuple *) malloc(sizeof(thm_tuple));
	cfg_local->sensors = NULL;
	cfg_local->fan = NULL;
	cfg_local->sensor_count = 0;
	if ((cfg_file = fopen(fname, "r")) == NULL) {
		showerr(fname);
		return NULL;
	}
	while ((line_len = getline (&line, &ll, cfg_file)) >= 0) {
		line_count++;
		if (sscanf(line, " sensor %1023c", buf) == 1) {
			cfg_local->sensors = (char **) realloc(
					cfg_local->sensors, (sensor_count+2) * sizeof(char *));
			buf[strcspn(buf, "\n")] = 0;
			cfg_local->sensors[sensor_count] = (char *) malloc(
					(strlen(buf)+1) * sizeof(char));
			strcpy(cfg_local->sensors[sensor_count], buf);
			cfg_local->sensor_count = ++sensor_count;
			cfg_local->sensors[sensor_count] = NULL;
		}
		else if (sscanf(line, " fan %1023c", buf) == 1) {
			buf[strcspn(buf, "\n")] = 0;
			if (cfg_local->fan == NULL) {
				cfg_local->fan = (char *) malloc((strlen(buf)+1) * sizeof(char));
				strcpy(cfg_local->fan, buf);
			}
			else {
				message(LOG_WARNING, MSG_WRN_CONF_FAN);
				return NULL;
			}
		}
		else if (sscanf(line, " ( %d , %d , %d ) ",
		 &cfg_local->items[i].level, &cfg_local->items[i].low,
		 &cfg_local->items[i].high) == 3) {
			i++;
			if ((cfg_local->items = realloc(cfg_local->items,
			 (i+1) * sizeof(thm_tuple))) == NULL) {
				showerr("Allocating memory for config");
				return NULL;
			}
			// check for correct ordering of fan levels in config
			if ((i > 2) && (cfg_local->items[i-1].level <=
			 cfg_local->items[i-2].level)) {
				if (nodaemon && chk_sanity && !quiet)
					fprintf(stderr, MSG_ERR_CONF_ORDER);
				if (!nodaemon && !quiet)
					syslog(LOG_ERR, MSG_ERR_CONF_ORDER);
				if (chk_sanity)
					return NULL;
			}
			if (cfg_local->items[i-1].high <= cfg_local->items[i-1].low) {
				if (nodaemon && chk_sanity && !quiet)
					fprintf(stderr, MSG_ERR_CONF_LOWHIGH);
				if (!nodaemon && !quiet)
					syslog(LOG_ERR, MSG_ERR_CONF_LOWHIGH);
				if (chk_sanity)
					return NULL;
			}
			cfg_local->max_idx = i-1;
		}
		free(line);
		line = NULL;
	}
	if (line != NULL) {
		free(line);
		line = NULL;
	}

	fclose(cfg_file);

	if (i <= 0) {
		message(LOG_ERR, MSG_ERR_CONF_PARSE);
		free_config();
		return NULL;
	}

	if (cfg_local->fan != NULL) {
		if (resume_is_safe) setfan = setfan_sysfs;
		else {
			setfan = setfan_sysfs_safe;
			message(LOG_WARNING, MSG_WRN_SYSFS_SAFE);
		}
		init_fan = init_fan_sysfs;
		uninit_fan = uninit_fan_sysfs;
	}
	else {
		setfan = setfan_ibm;
		init_fan = init_fan_ibm;
		uninit_fan = uninit_fan_ibm;
	}
	if (cfg_local->sensor_count > 0)
		get_temp = get_temp_sysfs;
	else
		get_temp = get_temp_ibm;

	return cfg_local;
}

void free_config(tf_config *cfg) {
	free(cfg->fan);
	if (cfg->sensor_count > 0) {
		int j;
		for (j=0; j < cfg->sensor_count; j++)
			free(cfg->sensors[j]);
		free(cfg->sensors);
	}
	free(cfg->items);
	free(cfg);
}


/***********************************************************
 * This is the main routine which periodically checks
 * temperatures and adjusts the fan according to config.
 ***********************************************************/
int fancontrol() {
	int last_temp=0, temp, lvl_idx=0, bias=0, diff=0, b_temp;
	int wt = watchdog_timeout, st = sleeptime;

	if (config == NULL) {
		message(LOG_ERR, MSG_ERR_CONF_LOST);
		return -1;
	}
	else if (!quiet) {
		int i;
		message(LOG_NOTICE, MSG_INF_CONFIG);
		for (i=0; i <= config->max_idx; i++) {
			message(LOG_INFO, MSG_INF_CONF_ITEM(
			 config->items[i].level, config->items[i].low,
			 config->items[i].high));
		}
	}

	init_fan();

	// Set initial fan level:
	lvl_idx = config->max_idx;
	b_temp = temp = get_temp();

	while ((temp <= config->items[lvl_idx].low) \
	 && (lvl_idx > 0)) lvl_idx--;
	setfan(config->items[lvl_idx].level);
#ifdef DEBUG
	message(LOG_DEBUG, MSG_DBG_T_STAT);
#endif

	/**********************************************
	 * Main loop. This is the actual fan control.
	 **********************************************/
	while(!interrupted) { // set by the signal handler

		// errcnt (global) is set by setfan, init_fan and get_temp.
		if (errcnt >= MAXERR) return errcnt;

		last_temp = temp; // detect temperature jumps
		sleep(st); // st is the variable sleeptime
		temp = get_temp();

		// If temperature increased by more than 2 °C since the
		// last cycle, we try to react quickly.
		diff = temp - last_temp;
		if (diff >= 2) {
			bias = (st * (diff-1)) * bias_level;
			if (st > 2) st = 2;
		}
		else if (st < sleeptime) st++;
		b_temp = temp + bias;
		if (b_temp >= config->items[lvl_idx].high) {
			while ((b_temp >= config->items[lvl_idx].high) \
			 && (lvl_idx <= config->max_idx)) lvl_idx++;
			setfan(config->items[lvl_idx].level);
#ifdef DEBUG
			message(LOG_DEBUG, MSG_DBG_T_STAT);
#endif
		}
		if (b_temp <= config->items[lvl_idx].low) {
			while ((b_temp <= config->items[lvl_idx].low) \
			 && (lvl_idx > 0)) lvl_idx--;
			setfan(config->items[lvl_idx].level);
			st = sleeptime;
#ifdef DEBUG
			message(LOG_DEBUG, MSG_DBG_T_STAT);
#endif
		}

		// slowly reduce the bias
		if (bias > 0) bias -= (bias_level + 0.1f) * 4;
		else (bias = 0);

		// Write current fan level to IBM_FAN one cycle before the watchdog
		// timeout ends, to let it know we're alive.
		if ( (wt -= st) <= st ) {
			setfan(config->items[lvl_idx].level);
			wt = watchdog_timeout;
		}
	}
	return 0;
}

void sigHandler(int signum) {
	switch(signum) {
	case SIGHUP:
		interrupted = signum;
		break;
	case SIGINT:
	case SIGTERM:
		interrupted = signum;
		break;
	}
}


/***************************************************************
 * Main function:
 * Scan for arguments, set options and initialize signal handler
 ***************************************************************/
int main(int argc, char **argv) {
	int opt, ret, fork_ret;
	char *invalid = "";
	struct sigaction handler;

	sleeptime = 5;
	quiet = 0;
	interrupted = 0;
	chk_sanity = 1;
	bias_level = 0.5f;
	ret = 0;
	config_file = "/etc/thinkfan.conf";
	nodaemon = 0;
	errcnt = 0;
	oldpwm = NULL;
	resume_is_safe = 0;

	openlog("thinkfan", LOG_CONS, LOG_USER);
	syslog(LOG_INFO, "thinkfan " VERSION " starting...");

	interrupted = 0;
	memset(&handler, 0, sizeof(handler));
	handler.sa_handler = sigHandler;
	if (sigaction(SIGHUP, &handler, NULL) \
	 || sigaction(SIGINT, &handler, NULL) \
	 || sigaction(SIGTERM, &handler, NULL)) perror("sigaction");

	while ((opt = getopt(argc, argv, "c:s:b:hnqDz")) != -1) {
		switch(opt) {
		case 'h':
			fprintf(stderr, MSG_USAGE);
			return 0;
			break;
		case 'n':
			nodaemon = 1;
			break;
		case 'c':
			config_file = optarg;
			break;
		case 's':
			sleeptime = (unsigned int) strtol(optarg, &invalid, 0);
			if (*invalid != 0) {
				fprintf(stderr, MSG_ERR_OPT_S MSG_USAGE);
				return 1;
			}
			break;
		case 'q':
			quiet = 1;
			break;
		case 'D':
			chk_sanity = 0;
			break;
		case 'b':
			bias_level = strtof(optarg, &invalid);
			if (*invalid != 0) {
				fprintf(stderr, MSG_ERR_OPT_B MSG_USAGE);
				return 1;
			}
			if (bias_level >= 0 && bias_level <= 20)
				bias_level = 0.1f * bias_level;
			else {
				fprintf(stderr, MSG_ERR_OPT_B MSG_USAGE);
				return 1;
			}
			break;
		case 'z':
			resume_is_safe = !0;
			break;
		default:
			fprintf(stderr, MSG_USAGE);
			return 1;
		}
	}

	if (sleeptime > 15) {
		if (chk_sanity) {
			fprintf(stderr, MSG_WRN_SLEEPTIME_15);
			fprintf(stderr, MSG_INF_SANITY);
			return 1;
		}
		else if (!quiet) {
			fprintf(stderr, MSG_WRN_SLEEPTIME_15);
			fprintf(stderr, MSG_INF_INSANITY);
		}
	}
	else if (sleeptime < 1) {
		if (chk_sanity) {
			fprintf(stderr, MSG_WRN_SLEEPTIME_1);
			fprintf(stderr, MSG_INF_SANITY);
			return 1;
		}
		else if (!quiet) {
			fprintf(stderr, MSG_WRN_SLEEPTIME_1);
			fprintf(stderr, MSG_INF_INSANITY);
		}
	}
	watchdog_timeout = sleeptime * 6;

	FILE *fd;
	if (chk_sanity && ((fd = fopen(PID_FILE, "r")) != NULL)) {
		fclose(fd);
		fprintf(stderr, MSG_ERR_RUNNING);
		return RV_PIDFILE;
	}

	if (nodaemon) ret = run();
	else {
		if ((fork_ret = fork()) == 0) ret = run();
		else if (!quiet)
			fprintf(stderr, "Daemon PID: %d\n", fork_ret);
		if (fork_ret < 0) {
			perror("fork()");
			ret = 1;
		}
	}
	return ret;
}

/********************************************************************
 * Outer loop. Handles signal conditions, runtime cleanup and config
 * reloading
 ********************************************************************/
int run() {
	int ret = 0;
	tf_config *newconfig=NULL;
	FILE *pidfile;

	if ((config = readconfig(config_file)) == NULL) {
		message(LOG_ERR, MSG_ERR_CONF_NOFILE);
		return -1;
	}

	if ((pidfile = fopen(PID_FILE, "w+")) == NULL) {
		showerr(PID_FILE);
		return -2;
	}
	fprintf(pidfile, "%d\n", getpid());
	fclose(pidfile);

	while (1) {
		ret = fancontrol();
		if (ret != 0) {
			message(LOG_ERR, MSG_ERR_ERRCOUNT);
			break;
		}
		else if (interrupted == SIGHUP) {
			interrupted = 0;
			message(LOG_DEBUG, MSG_DBG_CONF_RELOAD);
			if ((newconfig = readconfig(config_file)) != NULL) {
				free_config(config);
				config = newconfig;
			}
			else {
				if (nodaemon) fprintf(stderr, MSG_ERR_CONF_RELOAD);
				else syslog(LOG_ERR, MSG_ERR_CONF_RELOAD);
			}
		}
		else if (SIGINT <= interrupted && interrupted <= SIGTERM) {
			message(LOG_WARNING, "\nCaught deadly signal. ");
			break;
		}
	}

	message(LOG_WARNING, MSG_INF_TERM);
	uninit_fan();
	free_config(config);
	unlink(PID_FILE);
	return ret;
}

