/********************************************************************
 * thinkfan.h: Global stuff.
 *
 * This work is licensed under a Creative Commons Attribution-Share Alike 3.0
 * United States License. See http://creativecommons.org/licenses/by-sa/3.0/us/
 * for details.
 *
 * This file is part of thinkfan. See thinkfan.c for further info.
 * ******************************************************************/

#ifndef THINKFAN_H

#define THINKFAN_H
#define _GNU_SOURCE

#include "messages.h"

#define VERSION "0.5"
#define MAXERR 3 // maximum number of errors when trying to set fan speed

#define ERR_T_GET     	-1
#define ERR_FAN_INIT   	-2
#define ERR_CONF_NOFILE	-3
#define ERR_CONF_LOST	-4
#define ERR_CONF_RELOAD	-5
#define ERR_PIDFILE	   	-6
#define ERR_FANFILE    	-7
#define RV_PIDFILE		4

#define showerr(cause) \
	int lasterr = errno; \
	if (nodaemon) perror(cause); \
	syslog(LOG_ERR, "%s: %s", cause, strerror(lasterr));

#define message(loglevel, message) \
	if (nodaemon && (!quiet || loglevel > LOG_WARNING)) \
		fprintf(stderr, message); \
	else syslog(loglevel, message);

typedef struct {
	int level; // this is the fan speed level.
	int low;   // lower temperature for stepping down to previous level.
	int high;  // upper temperature determines when to step to the next level.
} thm_tuple;

typedef struct {
	thm_tuple *items;
	int max_idx;
	char **sensors;
	int sensor_count;
	char *fan;
} tf_config;

int quiet, nodaemon, errcnt, resume_is_safe;
volatile int interrupted;
unsigned int sleeptime;
unsigned int watchdog_timeout;
unsigned int chk_sanity;
char *config_file;
float bias_level;
tf_config *config;
char *oldpwm; // old contents of pwm*_enable, used for uninit_fan

int run();
void free_config();
int (*get_temp)() = NULL;
void (*setfan)(int) = NULL;
void (*init_fan)() = NULL;
void (*uninit_fan)() = NULL;
#endif
